########################################################################
# Gnuplot script to visualize MHD Steepening problem.
# Useful variables:
#
# dtype = "dbl" / "flt" / "tab" selects the data type to be read;
# nvar  = "rho" / "vx1" / ... " selects the variable to be plotted
#         (Full list in "pluto.gp");
# nout_max = total number of output files;
#
# Note: the tab formats allows field lines to be plotted, but it 
#       can be obtained only in serial mode.
#
# Last modified: 13 Dec 2021 by A. Mignone (mignone@to.infn.it)
########################################################################
reset
dtype="tab"   # Data type format ("dbl" / "flt" / "tab", see pluto.ini)

# -----------------------------------------------
# 0. Add gnuplot path [MAC / Linux]
#    This won't probaly work on Windows...
# -----------------------------------------------
pluto_dir = system("echo $PLUTO_DIR")
gnuplot_lib = pluto_dir."/Tools/Gnuplot"
set loadpath gnuplot_lib

load "pluto.gp"
nout_max = 35   # Max number of output files 

# -----------------------------------------------
# 1. Set font size
# -----------------------------------------------
set xlabel "x"  font ", 12"
set ylabel "y"  font ", 12"
set  tics       font ", 12"
set xtics       font ", 12"
set ytics       font ", 12"
set title       font ", 12"


do for [nout = 0:nout_max] {
  datafile = sprintf ("data.%04d.tab",nout)

  set yrange[-0.2:1.5]
  set style line 1 lt 2 lw 2
  set xlabel "x" font "Lucida, 16"
  set title "MHD Steepening and Shock Formation" font "Lucida,24"
  set key font "Lucida, 14"

  plot    datafile  using 1:(column(rho+3)) title "Density"  w lp
  replot  datafile  using 1:(column(prs+3)) title "Pressure"  w lp
  #replot  fname using 1:vx  title "vx"
  replot  datafile  using 1:(column(Bx2+3))  title "By"  w lp
  replot  datafile  using 1:(column(vx2+3))  title "vy"  w lp
 
  pause 0.2
}

