# ---------------------------------------------------------
# Gnuplot script to produce animation for the mhd blast
# wave problem.
#
# Last Modified: A. Mignone (24 Nov 2021)
# ---------------------------------------------------------
reset
load "pluto.gp"  # This file is written directly from PLUTO by enabling
                 # the GNUPLOT macro in definitions.h

# ---------------------------------------------------------
# 0. Set Plotting Environment 
# ---------------------------------------------------------
set autoscale xfixmin
set autoscale xfixmax
set autoscale yfixmin
set autoscale yfixmax
set xlabel "x" font ",12"
set ylabel "y" font ",12"
set tics       font ",12"
set lmargin at screen 0.15
set rmargin at screen 0.8
set bmargin at screen 0.1
set tmargin at screen 0.9

set pm3d map
set palette @INFERNO

ywinsize = 640                      # Assume 640 pixel in y direction
xwinsize = ywinsize*nx1*1.3/nx2     # extra factor 1.3 account for cbar  
set term qt size xwinsize, ywinsize

# ---------------------------------------------------------
# 1. Plot Specifications
# ---------------------------------------------------------
nvar      = prs
var_name  = "Pressure"

# -- Start Plotting --

do for [nout = 0:4] {
  title    = sprintf ("%s, nout = %d",var_name,nout)
  set title title font ",12"

  datafile = sprintf ("data.%04d.flt",nout)
  splot datafile @BINARR
  pause 0.4
}


