/* ///////////////////////////////////////////////////////////////////// */
/*!
  \file
  \brief Rayleigh-Taylor instability setup for MHD.

*/
/* ///////////////////////////////////////////////////////////////////// */
#include "pluto.h"

static double g_grav = -0.1; /* Gravity */
/* ********************************************************************* */
void Init (double *v, double x, double y, double z)
/*
 *
 *********************************************************************** */
{
  static int first_call = 1;
  double rnd, Bc;
  double kmin = 2.0*CONST_PI/(g_domEnd[IDIR] - g_domBeg[IDIR]);
  
  if (first_call == 1){          /* Seed random number sequence */
    RandomSeed(time(NULL),0);   
    first_call = 0;
  }
  rnd = RandomNumber(-1,1);         /* Generate random number in (-1,1) */

  if (y < 0.0) v[RHO] = 1.0;
  else         v[RHO] = g_inputParam[ETA];

  v[PRS] = 1.0/g_gamma + v[RHO]*g_grav*y;  /* Hydrostatic balance */
  v[VX1] = v[VX2] = v[VX3] = 0.0;

/* ----------------------------------------------
    Add perturbation
   ---------------------------------------------- */

  v[VX2] = 1.e-2*rnd*exp(-y*y*200.0);

/* ----------------------------------------------
    Set magnetic field in units of Bc:
 
    Bc = sqrt(g*\rho_1/(2k))  
   ---------------------------------------------- */

  #if PHYSICS == MHD
  v[BX1] = g_inputParam[CHI]*sqrt(fabs(g_grav)/(2.0*kmin));
  v[BX2] = 0.0;
  v[BX3] = 0.0;
  v[AX1] = v[AX2] = 0.0;
  v[AX3] = y*v[BX1];
  #endif

}

/* ********************************************************************* */
void InitDomain (Data *d, Grid *grid)
/*! 
 * Assign initial condition by looping over the computational domain.
 * Called after the usual Init() function to assign initial conditions
 * on primitive variables.
 * Value assigned here will overwrite those prescribed during Init().
 *
 *
 *********************************************************************** */
{ }

/* ********************************************************************* */
void Analysis (const Data *d, Grid *grid)
/* 
 *
 *
 *********************************************************************** */
{ }

/* ********************************************************************* */
void UserDefBoundary (const Data *d, RBox *box, int side, Grid *grid) 
/*
 *
 * 
 *********************************************************************** */
{ }

#if (BODY_FORCE & VECTOR)
/* ************************************************************************ */
void BodyForceVector(double *v, double *g, double x1, double x2, double x3)
/*  
 *
 *
 *************************************************************************** */
{
  g[IDIR] = 0.0;
  g[JDIR] = g_grav;
  g[KDIR] = 0.0;
  #ifdef CTU 
   if      (x2 > g_domEnd[JDIR]) g[JDIR] *= -1.0;
   else if (x2 < g_domBeg[JDIR]) g[JDIR] *= -1.0;
  #endif
}
#endif

#if (BODY_FORCE & POTENTIAL)
/* ************************************************************************ */
double BodyForcePotential(double x1, double x2, double x3)
/*
 *
 *
 *************************************************************************** */
{
}
#endif
